/// <reference path="util.ts"/>
/// <reference path="../../../res/lib/jquery/jquery.d.ts"/>

declare var h5:any;

declare namespace itpro.sample.report {
	var controller: any;
}

(function ($) {

    var HAS_ERROR_INPUT = 'has-error';

    /**
     * ReportController定義
     * 
     * @name itpro.sample.report.controller.ReportController
     * @namespace
     */
    var reportController = {
        /**
         * コントローラ名
         * 
         * @type String
         * @memberOf itpro.sample.report.controller.ReportController
         */
        __name: 'itpro.sample.report.controller.ReportController',
        /**
         * ユーザ情報
         */
        _userInfo: null,
        /**
         * @memberOf itpro.sample.report.controller.ReportController
         */
        __ready: function () {
            // ユーザID,ユーザ名を追加
            this._userInfo = itpro.sample.report.utils.getLoginUserInfo();
            this.$find('.userName').text(this._userInfo.id);
            this.$find('.realName').text(this._userInfo.name);

            this._$msg = this.$find('.report-content').find('.msg');

            this._setDafaultWorkTime();

            // コントローラバインド時にトップへスクロールして、タイトルバーを隠す(モバイル用)
            h5.ui.scrollToTop();

            var req = itpro.sample.report.utils.getRequestParameters();
            if (req.date) {
                // 日付が指定されていた場合は、その日の日報を新規作成

                // 日付の初期値を設定
                // リクエストパラメータに日付が指定されていればそれ。
                this.$find('input[name="reportDate"]').val(
                         itpro.sample.report.utils.formatDateWithHyphen(req.date));
                return;
            }

            // 日付が指定されていない場合は今日の日報
            this.$find('input[name="reportDate"]').val(itpro.sample.report.utils.formatDateWithHyphen(new Date()));
        },

        /**
         * inputとtextarea要素のfocusoutイベントで入力内容の検証をします
         * 
         * @memberOf itpro.sample.report.controller.ReportControlle
         */
        'input, textarea focusout': function(context, $el) {
            var value = $el.val();
            var name = $el.attr('name');
            if (name == 'img') {
              return;
            }

            var $formGroup = $el.parents('.form-group');
            if (value == null || value == '') {
                // 入力内容が空のとき
                if ($formGroup.hasClass(HAS_ERROR_INPUT)) {
                    // すでにエラーが表示されていれば何もしない
                    return;
                }
                // Bootstrapのクラス(has-error)を使って入力項目を強調
                $formGroup.addClass(HAS_ERROR_INPUT);
                // ページの先頭のmsgクラスの要素にメッセージを追加
                var label = $formGroup.find('label').text();
                var $p = $('<p data-itpro-input-name="' +  name + '">');
                $p.append('<strong>' + label + 'を入力してください' + '</strong>');
                this._$msg.append($p);
            } else {
                // 入力されているとき
        // 入力項目を強調するクラスを除去(ついていなければ何もしない
                $formGroup.removeClass(HAS_ERROR_INPUT);
                // メッセージを削除
                 this._$msg.find('p[data-itpro-input-name="' + name + '"]').remove();
            }

            // エラーメッセージがあればメッセージ領域を表示
            if (this._$msg.children().length != 0) {
                this._$msg.show();
            } else {
                this._$msg.hide();
            }
        },

        'button.regist click': function (context, $el) {
            context.event.preventDefault();
            alert('登録しました');
        },
        /**
         * レポートオブジェクトを作成
         * 
         * @memberOf itpro.sample.report.controller.ReportController
         * @returns Object
         */
        _createReport: function () {
            var reportDate = this.$find('input[name="reportDate"]').val();
            var startTime = this.$find('input[name="startTime"]').val();
            var endTime = this.$find('input[name="endTime"]').val();
            var reportId = this._reportId;
            var userInfo = this._userInfo;
            var sections = [];
            this.$find('.edit .section').each(function () {
                var $this = $(this);
                var sectionId = $this.find('input[name="sectionId"]').val();
                var category = $this.find('input[name="category"]').val();
                var title = $this.find('input[name="title"]').val();
                var comment = $this.find('textarea[name="comment"]').val();
                var section = new itpro.sample.report.Section(sectionId, category, title, comment);
                sections.push(section);
            });
            return {
                userInfo: userInfo,
                reportDate: reportDate,
                startTime: startTime,
                endTime: endTime,
                sections: sections
            };
        },
        /**
         * デフォルトの勤務時間を設定する
         * 
         * @memberOf itpro.sample.report.controller.ReportController
         */
        _setDafaultWorkTime: function () {
            // 9:00～現在時刻をデフォルトの勤務時間として設定
            this.$find('input[name="startTime"]').val('09:00');
            this.$find('input[name="endTime"]').val(itpro.sample.report.utils.formatTime(new Date()));
        }
    };
    // 名前空間 itpro.sample.report.controller に ReportController を配置
    h5.core.expose(reportController);
})(jQuery);
$(function () {
    // 公開されているコントローラを、操作対象となるHTML要素に割り当てる
    h5.core.controller(document.body, itpro.sample.report.controller.ReportController);
});