declare var h5 :any;
    /**
     * @namespace itpro.sample.report
     */
    /**
     * @namespace itpro.sample.report.utils
     */

    class RequestParamerters {
      date: string;
      startTime: string;
      endTime: string;
    }

    namespace itpro.sample.report {
        export class Section {
            constructor (public sectionId: string, public category: string, public title: string, public comment: string) {
            }
        }

        export class User {
            constructor (public userId: string, public userName: string, public realName) {
            }
        }

        export namespace utils {
          /**
           * ログインしているユーザ情報を取得します。<br>
           * 未ログイン時は、このメソッドはnullを返します。
           *
           * @function
           * @name getLoginUserInfo
           * @memberOf itpro.sample.report.utils
           * @returns {Object} ログインしているユーザ情報
           */
          export function getLoginUserInfo(): User {
              // ダミーの値を返す
              return new User('0000001', 'yamada', '山田太郎');
          };

      /**
       * URLのパラメータをオブジェクトにシリアライズします
       *
       * @returns {キー名:値}のオブジェクトに変換されたパラメータ
       * @memberOf itpro.sample.report.utils
        */
      export function getRequestParameters(): RequestParamerters {
        var queryString =location.search.match(/\?(.+)/); 
        return !queryString ? new RequestParamerters() : this.convertQueryParameterToMap(queryString[1]);
      }

    /**
     * URLのパラメータをオブジェクトにシリアライズします
     * 
     * @returns {キー名:値}のオブジェクトに変換されたパラメータ
     * @memberOf itpro.sample.report.utils
     */
      export function convertQueryParameterToMap(queryString: String): RequestParamerters {
        var params: RequestParamerters = new RequestParamerters();
        if (queryString != null && queryString.length > 0) {
          var queries = queryString[1].split('&');

          for (var i = 0, len = queries.length; i < len; i++) {
            var pair = queries[i].split('=');

            if (pair.length === 0) {
              continue;
            }

            params[pair[0]] = decodeURIComponent(pair[1].replace(/\+/g, '%20'));
          }
        }
        return params;
      }

          /**
           * 日付をyyyy-mm-ddにフォーマット
           *
           * @memberOf itpro.sample.report.utils
           * @param date
           * @returns
           */
          export function formatDateWithHyphen(date: Date): string {
              var y = date.getFullYear();
              var m = date.getMonth() + 1;
              var mStr = m < 10 ? '0' + m : m.toString();
              var d = date.getDate();
              var dStr = d < 10 ? '0' + d : d.toString();
              return h5.u.str.format('{0}-{1}-{2}', y, mStr, dStr);
          }

          /**
           * 日付をyyyy/mm/ddにフォーマット
           *
           * @memberOf itpro.sample.report.utils
           * @param date
           * @returns
           */
          export function formatDateWithSlash (date: string): string {
              return itpro.sample.report.utils.formatDateWithHyphen(new Date(date)).replace(/-/g, '/');
          }

          /**
           * 時刻をhh:mmにフォーマット
           *
           * @memberOf itpro.sample.report.utils
           * @param {Date} date
           * @returns
           */
          export function formatTime (date: Date): string {
              var h = date.getHours();
              var m = date.getMinutes();
              return (h < 10 ? '0' : '') + h + ':' + (m < 10 ? '0' : '') + m;
          }
      }
}