// =============================================================================
// 業務画面 1ページに対応する、コントローラ、ロジックの定義を以下に記載する.
//
// h5.core.exposeを使って、明示的な公開を行わない限り、
// コードの定義は本ファイル内に閉じる.
// =============================================================================
(function() {
	// =========================================================================
	// このファイル内のみで有効な定数
	// =========================================================================

	// =========================================================================
	// このファイル内のみで有効な変数
	// =========================================================================
	var seq_pict_id = 0;
	var pref_pict_id = 'pict-';
	var pref_oparation_id = 'ope-';

	/**
	 * 画像の品質を定義(業務要件に従い適切なファイルサイズになる様に調整する).
	 */
	var quality = 20;

	// =========================================================================
	// このファイル内のみで有効な関数
	// =========================================================================
	function getNewPictId() {
		seq_pict_id = seq_pict_id + 1;
		window.localStorage.setItem("seq_pict_id", seq_pict_id);
		return pref_pict_id + seq_pict_id;
	}

	// =========================================================================
	// ロジックの定義
	// =========================================================================
	var pictureLogic = {

		/**
		 * 名前定義.
		 */
		__name: 'logic.PictureLogic',

		/**
		 * 初期化処理.
		 */
		init: function() {
		//			console.log("initialize pictureLogic");

		//			seq_pict_id = window.localStorage.getItem("seq_pict_id");
		//			if (seq_pict_id == null) {
		//				seq_pict_id = 0;
		//				window.localStorage.setItem("seq_pict_id", 0);
		//			} else {
		//				seq_pict_id = parseInt(seq_pict_id);
		//			}
		//			console.log("current seq_pict_id: " + seq_pict_id);
		},

		/**
		 * カメラから写真を取得する.
		 *
		 * @param quality 画像品質.
		 * @returns ディファードオブジェクト(非同期処理制御用).
		 */
		getPictureFromCamera: function(quality) {
			var dfd = this.deferred();

			navigator.camera.getPicture(function(imageData) {
				console.log("image size(kb): " + imageData.length / 1024.0);
				dfd.resolve(imageData);
			}, function(message) {
				dfd.reject(message);
			}, {
				quality: quality,
				destinationType: Camera.DestinationType.DATA_URL,
				encodingType: Camera.EncodingType.JPEG,
			});

			return dfd.promise();
		},

		/**
		 * アルバムから写真を取得する.
		 *
		 * @param quality 画像品質.
		 * @returns ディファードオブジェクト(非同期処理制御用).
		 */
		getPictureFromAlbum: function(quality) {
			var dfd = this.deferred();

			navigator.camera.getPicture(function(imageData) {
				dfd.resolve(imageData);
			}, function(message) {
				dfd.reject(message);
			}, {
				quality: quality,
				destinationType: Camera.DestinationType.DATA_URL,
				encodingType: Camera.EncodingType.JPEG,
				sourceType: Camera.PictureSourceType.PHOTOLIBRARY
			});

			return dfd.promise();
		},

		/**
		 * 写真をセーブする.
		 *
		 * @param pictId 写真ID.
		 * @param imageData イメージデータ.
		 * @returns ディファードオブジェクト(非同期処理制御用).
		 */
		save: function(pictId, imageData) {
			var dfd = this.deferred();

			//			window.localStorage.setItem(pictId, imageData);
			//			console.log('save picture: pictId = ' + pictId);

			dfd.resolve(); // localstorageにはtxがないので黙ってresolve
			return dfd.promise();
		},

		/**
		 * 操作ステータスを更新する.
		 *
		 * @param operationId 操作ID.
		 * @param status ステータス.
		 */
		updateStatus: function(operationId, status) {
			console.log("operation update status: id = " + operationId + ', status='
					+ JSON.stringify(status));
			window.localStorage.setItem(operationId, JSON.stringify(status));
		},

		/**
		 * 操作ステータスを取得する.
		 *
		 * @param operationId 操作ID.
		 * @returns ステータス.
		 */
		getOperationStatus: function(operationId) {
			var status = window.localStorage.getItem(operationId);
			if (status == null) {
				return {};
			}

			return JSON.parse(status);
		},
	};

	h5.core.expose(pictureLogic);

	// =========================================================================
	// コントローラ定義
	// =========================================================================
	/**
	 * @class
	 * @name controller.OperationReportController
	 */
	var registerController = {
		/**
		 * 名前定義.
		 *
		 * @memberOf controller.OperationReportController
		 */
		__name: 'controller.OperationReportController',

		/**
		 * ロジック定義.
		 *
		 * @memberOf controller.OperationReportController
		 */
		pictureLogic: pictureLogic,

		/**
		 * ライフサイクルイベント(初期化時).
		 *
		 * @memberOf controller.OperationReportController
		 */
		__ready: function(context) {
			// 画面(コンポーネント)ロード時に処理を行いたい場合はここに記述

			//			console.log('ready');

			this.pictureLogic.init();

			//			this.loadSavedData();
		},

		_angle: 0,

		_selectedImg: null,

		_$clone: null,

		//		/**
		//		 * セーブ済データをロードする.
		//		 */
		//		loadSavedData: function() {
		//			// 入力状態復帰
		//			$('.operationInputArea')
		//					.each(
		//							function(i) {
		//
		//								var operationId = $(this).data('operationId');
		//								console.log('load saved data for ' + pref_oparation_id
		//										+ operationId);
		//
		//								var status = window.localStorage.getItem(pref_oparation_id
		//										+ operationId);
		//								console.log('targe status: ' + status);
		//
		//								$(this)
		//										.find('.pictureGroup')
		//										.append(
		//												'<div class="inputWrapper"><img src="res/image/camera.png">'
		//														+ '<input type="file" accept="image/*" class="blank-picture" data-action="getPicture"></div>');
		//
		//								return true;
		//
		//							});
		//		},

		//		/**
		//		 * 実施が押された場合の処理
		//		 *
		//		 * @memberOf controller.OperationReportController
		//		 * @param context
		//		 */
		//		'[data-action="done"] click': function(context) {
		//			console.log('select done');
		//			var id = this._getOperationIdFromContext(context);
		//
		//			// 未実施理由選択リストボックスをdisableに
		//			$(context.event.currentTarget).parent('dd').find('select').attr('disabled', true);
		//
		//			// 状態保存
		//			var status = this.pictureLogic.getOperationStatus(pref_oparation_id + id);
		//			status.status = '0';
		//			status.reason = undefined;
		//
		//			this.pictureLogic.updateStatus(pref_oparation_id + id, status);
		//		},
		//
		//		/**
		//		 * 未実施が押された場合の処理
		//		 *
		//		 * @memberOf controller.OperationReportController
		//		 * @param context
		//		 */
		//		'[data-action="notDone"] click': function(context) {
		//			console.log("selecte undone;");
		//			var id = this._getOperationIdFromContext(context);
		//			var dd = $(context.event.currentTarget).parents('dd');
		//
		//			// 未実施理由選択リストボックスをenableに
		//			dd.find('select').removeAttr('disabled');
		//			// $(context.event.currentTarget).parent('dd').find('select').removeAttr('disabled');
		//
		//			// 状態保存
		//			var status = this.pictureLogic.getOperationStatus(pref_oparation_id + id);
		//			status.status = '1';
		//			status.reason = dd.find('select').children(':selected').val();
		//
		//			this.pictureLogic.updateStatus(pref_oparation_id + id, status);
		//		},
		//
		//		/**
		//		 * 未実施理由が変更された場合の処理.
		//		 *
		//		 * @param context
		//		 */
		//		'[data-action="changeReason"] change': function(context) {
		//			var id = this._getOperationIdFromContext(context);
		//			var value = $(context.event.currentTarget).val();
		//			console.log('change reason: ' + value);
		//
		//			var status = this.pictureLogic.getOperationStatus(pref_oparation_id + id);
		//			status.reason = value;
		//			this.pictureLogic.updateStatus(pref_oparation_id + id, status);
		//		},
		//
		//		/**
		//		 * コメントが記入された場合の処理.
		//		 *
		//		 * @param context
		//		 */
		//		'[data-action="inputComment"] keyup': function(context) {
		//			var id = this._getOperationIdFromContext(context);
		//			var value = $(context.event.currentTarget).val();
		//			console.log('change comment: ' + value);
		//
		//			var status = this.pictureLogic.getOperationStatus(pref_oparation_id + id);
		//			status.comment = value;
		//			this.pictureLogic.updateStatus(pref_oparation_id + id, status);
		//		},
		//
		//		/**
		//		 * 実働時間が変更された場合の処理.
		//		 *
		//		 * @param context
		//		 */
		//		'[data-action="updateTime"] change': function(context) {
		//			var id = this._getOperationIdFromContext(context);
		//
		//			var dd = $(context.event.currentTarget).parent('dd');
		//			var hour = dd.find('.operationResultHour').children(':selected').val();
		//			var min = dd.find('.operationResultMin').children(':selected').val();
		//
		//			var status = this.pictureLogic.getOperationStatus(pref_oparation_id + id);
		//			status.hour = hour;
		//			status.min = min;
		//
		//			this.pictureLogic.updateStatus(pref_oparation_id + id, status);
		//		},

		/**
		 * カメラ撮影を行う。
		 *
		 * @memberOf controller.OperationReportController
		 * @param context
		 */
		'[data-action="getPicture"] change': function(context, $el) {
			if (context.event.currentTarget.files.length === 0) {
				return;
			}
			console.log("controller: getPictrure");

			context.event.preventDefault(); // ページ先頭に移動する挙動を抑制

			var dfd = this.deferred();

			var indicator = this.indicator({
				target: document.body,
				message: 'プレビューを準備中...',
				promises: dfd.promise(),
			}).show();

			// プレビューがあれば除去
			var $preview = this.$find('.previewContainer');
			$preview.empty;
			$preview.remove();

			var reader = new FileReader();
			reader.onload = this.own(function(ev) {
				this._addPreview(context, reader.result, $el, dfd);
			});

			reader.readAsDataURL(context.event.currentTarget.files[0]);
		},

		_addPreview: function(context, imageData, $el, dfd) {

			var $img = $('<img>');
			var img = $img.get(0);

			img.onload = this
					.own(function() {
						// 写真、選択した画像を覚えておく
						this._selectedImg = img;

						// プレビューを生成
						var $previewContainer = $('<div class="previewContainer"></div>');

						var $canvasContainer = $('<div class="canvasContainer"><canvas width="550px" height="412.5px"></canvas></div>');

						var $btnContainer = $('<div class="btnContainer"></div>');

						var $btns = $('<div class="angle-btn-wrap"><button class="btn-angle btn-0">0度</button>'
								+ '<button class="btn-angle btn-90">90度</button>'
								+ '<button class="btn-angle btn-180">180度</button>'
								+ '<button class="btn-angle btn-270">270度</button>'
								+ '<button class="btn-angle btn-90-con">90度ずつ</button></div>'
								+ '<div class="handle-btn-wrap"><button class="btn-angle setAngle">決定</button>'
								+ '<button class="btn-angle cancel">キャンセル</button></div>');

						$previewContainer.append($canvasContainer);
						$canvasContainer.append($btnContainer);
						$btnContainer.append($btns);

						$el.parents('.operationInputArea').append($previewContainer);

						var canvas = $previewContainer.find('canvas').get(0);

						this._$clone = $img.clone();
						this._$clone.removeClass('picture');

						this.img2Canvas(this._$clone.get(0), canvas);

						$previewContainer.css('display', 'block');// プレビュー表示

						dfd.resolve();// インジケータを非表示
					});

			$img.attr('src', imageData);
		},

		img2Canvas: function(img, canvas) {
			var w = canvas.width;
			var h = canvas.height;
			var sw = img.naturalWidth;
			var sh = img.naturalHeight;

			var ctx = canvas.getContext('2d');

			var srcAspectRatio = sh / sw;
			var canvasAspectRatio = h / w;

			var dx, dy, dw, dh;
			if (canvasAspectRatio >= srcAspectRatio) {
				// canvasの方が横に長い場合
				dw = w;
				dh = w * srcAspectRatio;
				dx = 0;
				dy = (h - dh) / 2;
			} else {
				// canvasの方が縦に長い場合
				dh = h;
				dw = h / srcAspectRatio;
				dx = (w - dw) / 2;
				dy = 0;
			}

			ctx.drawImage(img, 0, 0, sw, sh, dx, dy, dw, dh);
		},


		/**
		 * 90度、270度回転するとき画像の端が欠けないよう縮小させる
		 *
		 * @param img
		 * @param canvas
		 */
		_scaleImage: function(canvas) {
			var ctx = canvas.getContext('2d');

			var w = canvas.width;
			var h = canvas.height;

			var aspect = h / w;

			// 長辺がcanvasに収まるよう変形
			ctx.scale(aspect, aspect);
		},


		'.btn-0 click': function(context) {
			context.event.preventDefault();

			this._angle = 0;
			this._rotateImage(0);
		},


		'.btn-90 click': function(context) {
			context.event.preventDefault();

			this._angle = 90;
			this._rotateImage(90);
		},


		'.btn-180 click': function(context) {
			context.event.preventDefault();

			this._angle = 180;
			this._rotateImage(180);
		},


		'.btn-270 click': function(context) {
			context.event.preventDefault();

			this._angle = 270;
			this._rotateImage(270);
		},


		'.btn-90-con click': function(context) {
			context.event.preventDefault();

			if (this._angle === 270) {
				this._angle = 0;
			} else {
				this._angle += 90;
			}
			this._rotateImage(this._angle);
		},


		_rotateImage: function(rad) {
			var canvas = this.$find('.previewContainer canvas').get(0);
			var ctx = canvas.getContext('2d');

			var w = canvas.width;
			var h = canvas.height;

			var img = this._$clone.get(0);

			var theta = rad;

			rad = rad * Math.PI / 180;

			// canvasを回転させる
			ctx.clearRect(0, 0, w, h);
			ctx.save();
			ctx.translate(w / 2, h / 2);
			ctx.rotate(rad);

			if (theta === 90 || theta === 270) {
				this._scaleImage(canvas);
			}

			ctx.translate(-1 * w / 2, -1 * h / 2);

			this.img2Canvas(img, canvas);

			ctx.restore();
		},


		/**
		 * 報告内容の写真カラムに画像を表示する。
		 *
		 * @param context
		 * @param $el
		 */
		'.setAngle click': function(context, $el) {
			context.event.preventDefault();

			// プレビューのcanvasのimageDataではsketchでぼやけるので
			// キャッシュした元のデータを複製し、回転させたデータを取得する
			var sw = this._selectedImg.width;
			var sh = this._selectedImg.height;

			// 元画像のサイズでは処理が重いので固定値にする
			var w = 1000;// canvasのwidth
			var aspect = w / sw;
			var h = sh * aspect;// canvasのheight

			var canvas = $('<canvas width="' + w + '" height="' + h + '">').get(0);
			var ctx = canvas.getContext('2d');

			var $img = $(this._selectedImg);
			$img.removeClass('picture');
			var img = $img.get(0);

			var theta = this._angle;
			var rad = theta * Math.PI / 180;

			// canvasを回転させる
			ctx.clearRect(0, 0, w, h);
			ctx.save();
			ctx.translate(w / 2, h / 2);
			ctx.rotate(rad);

			if (theta === 90 || theta === 270) {
				this._scaleImage(canvas);
			}

			ctx.translate(-1 * w / 2, -1 * h / 2);

			this.img2Canvas(img, canvas);

			ctx.restore();

			var imageData = canvas.toDataURL('image/png');


			this._addPicture($el, imageData);
		},


		'.cancel click': function(context, $el) {
			context.event.preventDefault();

			this.$find('[data-action="getPicture"]').val('');

			// プレビューを除去
			var $container = this.$find('.previewContainer');
			$container.empty();
			$container.remove();
			this._angle = 0;
		},


		_addPicture: function($el, imageData) {
			var $area = $el.parents('.operationInputArea');

			var $resultCamera = $area.find('.operationResultArea').children(
					'.operationResultCamera');
			var $pictureGroup = $resultCamera.find('.pictureGroup');

			var $img = $('<img class="picture">').attr('src', imageData);
			//			var $img = $(img);
			//			$img.addClass('picture');

			// 選択した画像を報告内容に追加
			$pictureGroup.append($img);
			// 削除ボタンを追加
			$pictureGroup.append('<div class="deletePicture">×</div>');

			this.$find('[data-action="getPicture"]').val('');

			// プレビューを除去
			var $container = this.$find('.previewContainer');
			$container.empty();
			$container.remove();
			this._angle = 0;

			this._selectedImg = null;
			this._$clone = null;
		},


		//		/**
		//		 * @param operationInputArea
		//		 */
		//		clearOperationInputArea: function(operationInputArea) {
		//			var inputArea = $(operationInputArea);
		//			var id = inputArea.data('operationId');
		//			console.log('clear ope-' + id);
		//
		//			// 実施状況
		//			inputArea.find('input:radio').attr("checked", false);
		//			inputArea.find('.operationResultInputContents').find('select').attr('disabled', true)
		//					.val('0');
		//
		//			// コメント
		//			inputArea.find('textarea').val('');
		//
		//			// 写真
		//			inputArea.find('.pictureGroup').empty();
		//
		//			// 実働時間
		//			var timeRoot = inputArea.find('.operationResultTimeContents');
		//			timeRoot.find('.operationResultHour').val('0');
		//			timeRoot.find('.operationResultMin').val('00');
		//
		//		},

		_getOperationIdFromContext: function(context) {
			return $(context.event.currentTarget).parents('.operationInputArea')
					.data('operationId');
		}

	};

	/**
	 * コントローラ定義を公開.
	 */
	h5.core.expose(registerController);

	//	// LocalStrageクリア用コントローラ
	//	var localStorageController = {
	//
	//		/**
	//		 * 名前定義.
	//		 */
	//		__name: 'controller.localStorageController',
	//
	//		/**
	//		 * ライフサイクル定義(初期化処理).
	//		 *
	//		 * @param context
	//		 */
	//		__ready: function(context) {
	//			console.log("redy my controller");
	//		}
	//
	//	};
	//
	//	/**
	//	 * コントローラ定義を公開.
	//	 */
	//	h5.core.expose(localStorageController);

})();