(function($) {

	var REGIST_URL = 'regist';
	
	// ダミーajax
	var ajax = function() {
		var dfd = h5.async.deferred()
		setTimeout(function() {
			dfd.resolve();
		}, 1000);
		return dfd.promise();
	};

	var reportLogic = {
		/**
		 * ロジック名
		 *
		 * @memberOf itpro.sample.report.logic.ReportLogic
		 */
		__name: 'itpro.sample.report.logic.ReportLogic',

		register: function(reportData) {
			return ajax(REGIST_URL, {
				type: 'post',
				dataType: 'json',
				data: reportData
			});
		}
	};

	h5.core.expose(reportLogic);
})(jQuery);

(function($) {
	var HAS_ERROR_INPUT = 'has-error';
	var utils = itpro.sample.report.utils;
	/**
	 * ReportController定義
	 *
	 * @name itpro.sample.report.controller.ReportController
	 * @namespace
	 */
	var reportController = {

		/**
		 * コントローラ名
		 *
		 * @type String
		 * @memberOf itpro.sample.report.controller.ReportController
		 */
		__name: 'itpro.sample.report.controller.ReportController',

		_reportLogic: itpro.sample.report.logic.ReportLogic,
		
		/**
		 * テンプレートファイル
		 *
		 * @type String
		 * @memberOf itpro.sample.report.controller.ReportController
		 */
		__templates: 'template/report.ejs',

		/**
		 * お絵かきコントローラ
		 *
		 * @type {Controller}
		 * @memberOf itpro.sample.report.controller.ReportController
		 */
		artboardWrapperController: itpro.sample.report.controller.ArtboardWrapperController,

		/**
		 * 画像編集コントローラ
		 */
		operationController: controller.OperationReportController,

		__meta: {
			artboardWrapperController: {
				rootElement: '.sketch-canvas-wrapper'
			}
		},

		/**
		 * セクションのひな形（追加時に使用）
		 */
		_$sectionCopy: null,

		/**
		 * 画像アップロードフォーム
		 *
		 * @memberOf itpro.sample.report.controller.ReportController
		 */
		_$imgInputForm: null,

		/**
		 * 画像ID用のシーケンス
		 */
		_imgIdSeq: 0,

		/**
		 * セクション用シーケンス
		 */
		_sectionSequence: 1,

		/**
		 * ユーザ情報
		 */
		_userInfo: null,

		/**
		 * レポートID
		 */
		_reportId: null,

		/**
		 * 削除するセクションID
		 */
		_deleteSectionIds: [],

		/**
		 * 削除する画像
		 */
		_deleteImgUrls: [],

		/**
		 * @memberOf itpro.sample.report.controller.ReportController
		 */
		__ready: function() {
			// ユーザID,ユーザ名を追加
			this._userInfo = utils.getLoginUserInfo();
			this.$find('.userName').text(this._userInfo.id);
			this.$find('.realName').text(this._userInfo.name);
			this._$msg = this.$find('.report-content').find('.msg');
			this._$indicator = this.$find('.indicator-wrap');
			
			// コントローラバインド時にトップへスクロールして、タイトルバーを隠す(モバイル用)
			h5.ui.scrollToTop();

			// 日付が指定されていない場合は今日の日報
			this.$find('input[name="reportDate"]').val(utils.formatDateWithHyphen(new Date()));
			this._setDafaultWorkTime();

			this._$sectionCopy = this.$find('.section').clone();

			this._initSection();
		},

		/**
		 * inputとtextarea要素のfocusoutイベントで入力内容の検証をします
		 *
		 * @memberOf itpro.sample.report.controller.ReportControlle
		 */
		'input, textarea focusout': function(context, $el) {
			var value = $el.val();
			var name = $el.attr('name');
			if (name == 'img') {
				return;
			}

			var $formGroup = $el.parents('.form-group');
			if (value == null || value == '') {
				// 入力内容が空のとき
				if ($formGroup.hasClass(HAS_ERROR_INPUT)) {
					// すでにエラーが表示されていれば何もしない
					return;
				}
				// Bootstrapのクラスを使って赤い枠を表示
				$formGroup.addClass(HAS_ERROR_INPUT);
				// ページの先頭のmsgクラスの要素にメッセージを追加s
				var label = $formGroup.find('label').text();
				var $p = $('<p data-itpro-input-name="' + name + '">');
				$p.append('<strong>' + label + 'を入力してください' + '</strong>');
				this._$msg.append($p);
			} else {
				// 入力されているとき
				// 赤枠を表示するクラスを除去（ついていなければ何もしない）
				$formGroup.removeClass(HAS_ERROR_INPUT);
				// メッセージを削除
				this._$msg.find('p[data-itpro-input-name="' + name + '"]').remove();
			}
			// エラーメッセージがあればメッセージ領域を表示
			if (this._$msg.children().length != 0) {
				this._$msg.show();
			} else {
				this._$msg.hide();
			}
		},

		/**
		 * セクションの追加
		 */
		'.section-append click': function() {
			this._appendSection();
		},

		/**
		 * @memberOf itpro.sample.report.controller.ReportController
		 */
		_appendSection: function(section) {
			this.$find('.section-wrap').append(this._$sectionCopy.clone());
			return this._initSection(section);
		},

		_initSection: function(section) {
			var $section = this.$find('.section:last');
			$section.find('.report_num').text(this._sectionSequence);
			var collapseId = 'section-collapse' + this._sectionSequence;
			$section.find('.panel-title > a').attr('href', '#' + collapseId);
			$section.find('.panel-collapse').attr('id', collapseId);
			this._sectionSequence++;

			// 画像ファイルアップロードフォームをコピーしておく
			if (!this._$imgInputForm) {
				this._$imgInputForm = this.$find('.img-input-wrap:first').children().clone();
			}

			// sectionがないとき(=新規追加時)
			if (!section) {
				// 位置情報を取得してinput要素の値に設定する
				var $placeInput = this.$find('.edit .section:last input[name="place"]');
				var geoPromise = h5.api.geo.getCurrentPosition({
					timeout: 5000
				}).always(function(result) {
					var coords = result.coords;
					// 取得できなかった場合は空にしてメッセージを表示
					if (coords) {
						$placeInput.val(coords.latitude + ',' + coords.longitude);
					} else {
						alert('現在位置を取得できませんでした');
					}
				});
				// 位置情報取得時にインジケータを出しておく
				this.indicator({
					target: $placeInput,
					message: '位置情報取得中',
					promises: geoPromise
				}).show();

				// sectionがないとき(=新規追加時)は以降の処理は不要
				return $section;
			}

			// セクション情報が渡されたら、フォーム、画像を設定
			// 画像を設定
			if (section.urlList && section.urlList.length) {
				var base = itpro.sample.report.constant.URL_BASE_IMG;
				$section.find('.attached-img').html('');
				for (var i = 0, l = section.urlList.length; i < l; i++) {
					var url = section.urlList[i];
					this.view.append($section.find('.attached-img'), 'imgThumbnail', {
						param: {
							src: base + url,
							url: url
						}
					});
				}
			}

			if (section.sectionId) {
				$section.find('input[name="sectionId"]').val(section.sectionId);
			}
			if (section.category) {
				$section.find('select[name="category"]').val(section.category);
			}
			if (section.comment) {
				$section.find('textarea[name="comment"]').val(section.comment);
			}
			if (section.place) {
				$section.find('input[name="place"]').val(section.place);
			}
			return $section;
		},

		/**
		 *
		 */
		'.section-delete click': function(context, $el) {
			var $section = $el.parents('.section:first');
			var sectionId = $section.find('input[name="sectionId"]').val();
			if (sectionId) {
				this._deleteSectionIds.push(sectionId);
			}
			$section.remove();
			if (!this.$find('.section').length) {
				this._appendSection();
			}
		},

		/**
		 * プレビューボタンクリック
		 *
		 * @memberOf itpro.sample.report.controller.ReportController
		 * @param context
		 */
		'button.showpreview click': function(context) {
			h5.ui.scrollToTop();

			// 切替
			this.$find('.edit').css('display', 'none');
			this.$find('.preview').css('display', 'block');

			// 編集結果を取得して反映
			var report = this._createReport();
			this.view.update('.preview', 'preview', report);
		},

		/**
		 * 編集ボタンクリック
		 *
		 * @memberOf itpro.sample.report.controlleer.ReportController
		 * @param context
		 */
		'button.showedit click': function(context) {
			h5.ui.scrollToTop();

			// 切替
			this.$find('.edit').css('display', 'block');
			this.$find('.preview').css('display', 'none');

			this.$find('.preview').html('');
		},

		/**
		 * 画像ファイル削除
		 *
		 * @memberOf itpro.sample.report.controller.ReportController
		 * @param context
		 * @param $el
		 */
		'.deletePicture click': function(context, $el) {
			context.event.stopImmediatePropagation();
			if (!confirm('画像を削除しますか？')) {
				return;
			}
			$el.prev('.picture').remove();
			$el.remove();
		},

		/**
		 * 画像サムネイルクリック
		 */
		'.picture click': function(context, $el) {
			// ArtboardWritpro.sample.reporterControllerに渡す
			this.$find('.sketch-canvas-wrapper').css('display', 'block');
			this.artboardWrapperController.setPicture($el[0]);
		},
		
		/**
		 * 日報をサーバに登録する
		 *
		 * @memberOf itpro.sample.report.controller.ReportControlle
		 */
		'.regist click': function(context, $el) {
			// form内でボタンをclickしたときのデフォルトの挙動（画面の遷移）をキャンセル
			context.event.preventDefault();

			// 送信データを生成し、サーバに送信
			var reportData = this._createReport();
			var promise = this._reportLogic.register(reportData).always(function(data) {
				// 成功時のメッセージを表示
				alert('日報を登録しました');
			});

			this.indicator({
				message: '登録中...',
				promises: promise
			}).show();
		},

		/**
		 * レポートオブジェクトを作成
		 *
		 * @memberOf itpro.sample.report.controller.ReportController
		 * @returns Object
		 */
		_createReport: function() {
			var reportDate = this.$find('input[name="reportDate"]').val();
			var startTime = this.$find('input[name="startTime"]').val();
			var endTime = this.$find('input[name="endTime"]').val();
			var reportId = this._reportId;

			var sections = [];
			this.$find('.edit .section').each(function() {
				var $this = $(this);
				var sectionId = $this.find('input[name="sectionId"]').val();
				var place = $this.find('input[name="place"]').val();
				var category = $this.find('select[name="category"]').val();
				var comment = $this.find('textarea[name="comment"]').val();
				var imgs = [];
				$this.find('.picture').each(function() {
					imgs.push(this.src);
				});
				var urlList = [];
				$this.find('input[name="urlList"]').each(function() {
					urlList.push($(this).val());
				});
				sections.push({
					sectionId: sectionId,
					dailyReportId: reportId,
					place: place,
					category: category,
					comment: comment,
					imgs: imgs,
					urlList: urlList
				});
			});
			return {
				userInfo: this._userInfo,
				reportId: reportId,
				reportDate: reportDate,
				startTime: startTime,
				endTime: endTime,
				sections: sections
			};
		},

		/**
		 * デフォルトの勤務時間を設定する
		 *
		 * @memberOf itpro.sample.report.controller.ReportController
		 */
		_setDafaultWorkTime: function() {
			// 9:00～現在時刻をデフォルトの勤務時間として設定
			this.$find('input[name="startTime"]').val('09:00');
			this.$find('input[name="endTime"]').val(utils.formatTime(new Date()));
		}
	};

	// 名前空間 itpro.sample.report.controller に ReportController を配置
	h5.core.expose(reportController);
})(jQuery);