(function() {

  var report = itpro.sample.report;
  var utils = report.utils;

  QUnit.module('Section');

  QUnit.test('セクション情報を生成できる', function(assert) {
    var expected = {
        sectionId: 0,
        category: 'カテゴリ',
        title: 'タイトル',
        comment: 'コメント'
    };

    var actual = new report.Section(0, 'カテゴリ', 'タイトル', 'コメント');
    assert.equal(actual.sectionId, expected.sectionId);
    assert.equal(actual.category, expected.category);
    assert.equal(actual.title, expected.title);
    assert.equal(actual.comment, expected.comment);
  });

  QUnit.module('getLoginUserInfo');

  QUnit.test('ユーザ情報が取得できる', function(assert) {
    var actual = utils.getLoginUserInfo();
    assert.equal(actual.userId, '0000001');
    assert.equal(actual.userName, 'yamada');
    assert.equal(actual.realName, '山田太郎');
  });

  QUnit.module('formatDateWithHyphen');

  QUnit.test('ハイフン繋ぎの日付の文字列が取得できる', function(assert) {
    var expect = '2016-10-22';
    var date = new Date(expect);
    var actual = utils.formatDateWithHyphen(date);
    assert.equal(actual, expect);
  });

  QUnit.test('1桁の月でも0埋めした文字列が取得できる', function(assert) {
    var expect = '2016-08-22';
    var date = new Date('2016-8-22');
    var actual = utils.formatDateWithHyphen(date);
    assert.equal(actual, expect);
  });

  QUnit.test('1桁の日にちでも0埋めした文字列が取得できる', function(assert) {
    var expect = '2016-10-01';
    var date = new Date('2016-10-1');
    var actual = utils.formatDateWithHyphen(date);
    assert.equal(actual, expect);
  });

  QUnit.module('formatDateWithSlach');
  QUnit.test('スラッシュ繋ぎの日付の文字列が取得できる', function(assert) {
    var expect = '2016/10/22';
    var date = new Date(expect);
    var actual = utils.formatDateWithSlash(date);
    assert.equal(actual, expect);
  });

  QUnit.test('ハイフン繋ぎの日付の文字列が取得できる', function(assert) {
    var expect = '2016-10-22';
    var date = new Date(expect);
    var actual = utils.formatDateWithHyphen(date);
    assert.equal(actual, expect);
  });

  QUnit.module('formatTime');

  QUnit.test('時刻の文字列が取得できる', function(assert) {
    var expect = '12:30';
    var date = new Date('2016-10-19 12:30');
    var actual = utils.formatTime(date);
    assert.equal(actual, expect);
  });

  QUnit.test('時間が1桁の場合、0埋めして表示される', function(assert) {
    var expect = '09:30';
    var date = new Date('2016-10-19 9:30');
    var actual = utils.formatTime(date);
    assert.equal(actual, expect);
  });

  QUnit.test('分が1桁の場合、0埋めして表示される', function(assert) {
    var expect = '12:05';
    var date = new Date('2016-10-19 12:5');
    var actual = utils.formatTime(date);
    assert.equal(actual, expect);
  });

  QUnit.module('convertQueryParameterToMap');

  QUnit.test('パラメータが１つの場合に、Mapに分解される', function(assert) {
    var query = 'a=1';
    var expected = {
        a: '1'
    };
    var actual = utils.convertQueryParameterToMap(query);
    assert.deepEqual(actual, expected);
  });

  QUnit.test('パラメータが2つの場合に、Mapに分解される', function(assert) {
    var query = 'a=1&b=2';
    var expected = {
        a: '1',
        b: '2'
    };
    var actual = utils.convertQueryParameterToMap(query);
    assert.deepEqual(actual, expected);
  });

  QUnit.test('クエリが&で終わっている場合、最後の&を無視して分解される', function(assert) {
    var query = 'a=1&b=2&';
    var expected = {
        a: '1',
        b: '2'
    };
    var actual = utils.convertQueryParameterToMap(query);
    assert.deepEqual(actual, expected);
  });

})();
